

/****************************************************************
*								*
*                  Welcome to Hi-Tech PICC Workshop		*          
*								*
*****************************************************************
*								*
*   Filename:       config.c					*
*   Date:           07/27/2000					*
*   File Version:   1.00					*
*								*
*   Compiler:       Hi-Tech PIC C Compiler V7.86 PL3		*
*								*
*   Author:         Richard Yang					*
*   Company:        Microchip Technology Taiwan				*
*								*
*****************************************************************
*								*
*   Files required:						*
*								*
*                   config.c					*
*                   init.c					*
*                   delay.c      (Hi-Tech file)			*
*								*
*                   pic.h        (Hi-Tech file)			*
*                   delay.h      (Hi-Tech file)			*
*                   powerup.as   (Hi-Tech file)                 *
*								*
*****************************************************************
*								*
*   Notes:							*
*								*
*   Device Fosc -> 4.00MHz (Clock supplied by target)		*
*								*
*****************************************************************/

#include <pic.h>				// processor if/def file
#include "c:\ht-pic\samples\delay.h"		// 
//#include "cnfig77.h"				// processor configuration file
#include <stdio.h>

__CONFIG  ( HS & WDTDIS & PWRTEN & BORDIS & LVPDIS & UNPROTECT ); 
__IDLOC  ( 12AB );
__EEPROM_DATA(0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07);
__EEPROM_DATA(0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f);
__EEPROM_DATA('H','i','-','T','e','c','h',0x00,);


// ========================================================================================
// **** Establish PIC16C77 Configuration Word
// **** == HS Oscillator Mode
// **** == Brown-Out Detect Enabled
// **** == Watch-Dog Timer Off
// **** == Code Protect Off

//__CONFIG  ( HS_OSC & BODEN_ON & WDT_OFF & CP_OFF ); 

// ******************************[ FUNCTION PROTOTYPES ]***********************************

extern void Init_Usart( unsigned long int baud, unsigned char txset, unsigned char rcset );
extern void Init_Adc( void );
extern void Init_Timer1(void);

// *****************************[ VARIABLE DECLARATION ]***********************************

bank1 char ram_msg[] = "RAM Message"; 		// message in RAM
const char rom_msg[] = "ROM Message"; 		// message in ROM

const char *msgptr;				// define a const type pointer
unsigned char PotVal;				// Declare variable for Potiometer Value
unsigned long DutyCycle;			// Declare a long var for computing Duty Cycle of PWM1

// *******************************[ SYSTEM DEFINITIONS ]***********************************

#define POT	(0)				// Potentiometer is Analog Channel 0
#define TEMP	(1)				// Temperature Sense is Analog Channel 1
#define LIGHT	(2)				// Light Sense is Analog Channel 2

// **********************************[ FUNCTIONS ]*****************************************


/*******************************
       New Startup Routine
********************************/

void mystartup( void )
{
    // **** Add TRISD & PORTD initialization here ****
    PORTD=0x00;                           	// Initialize PORTD latch states
    TRISD=0x00;                            	// Set PORTD as outputs

    asm( "movlw	start>>8");			// Method for embedding assembly
    asm( "movwf	_PCLATH");
    asm( "goto	start & 7FFh");  	    	// return control back to program
}


/***********************************
      Interrupt Service Routine
************************************/

interrupt isr(void)
{
	if (TMR1IF)				// Is TMR1 Overflow source of interrupt?
	{					// Yes,
		PORTD++;			//	Increment PORTD
		TMR1H = 0xC0;			//	Reload TMR1 High Byte for 0.5 second Interrupt
		TMR1IF = 0;			//	Clear TMR1 Interrupt Flag
		printf("%d\r\n",DutyCycle);	//	Display value of Potentiometer A/D Result
	};
}

/***********************************
        Convert A/D Channel
************************************/

unsigned char A2D( unsigned char channel )
{
	unsigned char temp;			// Temp storage

	temp = channel << 3;			// Shift channel value three bits left
	ADCON0 &= 0xC7;				// Clear channel select bits in ADCON0
	ADCON0 |= temp;				// Logically OR shifted channel value into place in ADCON0
 
	ADGO = 1;				// Initiate conversion on selected channel
	while (ADGO) ;				// Poll ADGO bit until it clears, indicating conv. done
	return (ADRESH);			// Return result of A/D conversion
};

/************************************
   PUT CHARACTER FUNCTION for USART
*************************************/

void putch(unsigned char data)
{
	while(!TRMT) ;				// Wait until no characters are pending transmission
	
	TXREG = data;				// Place character in transmit register
};


/*****************************
    INITIALIZE SYSTEM
*****************************/

void Init_System(void)
{
	PORTD = 0x00;				// Initialize PORTD latch states
	TRISD = 0x00;				// set PORTD as outputs

	TRISC = 0b10111111;			// Initialize PORTC pin usage (RX input, all others output)

	Init_Usart(19200, 0x24, 0x90);		// Initialize USART peripheral (check function details for parameters)

	msgptr = rom_msg;			// Initialize character pointer variable

	Init_Adc();				// Initialize A/D Module

	Init_Timer1();				// Initialize TMR1 Module

	PEIE = 1;				// Enable Peripheral Interrupts

	GIE = 1;				// Enable Global Interrupts
};
	
/*******************************************************************
  MAIN PROGRAM BEGINS HERE
********************************************************************/

void main( void )
{
	Init_System();				// Initialize System Function and Variables

	while(1)  				// infinite loop
	{
		PotVal = A2D(POT);		// Obtain value of potiometer
	};
}
