// MID_LCD.C
// Contain all the necessary LCD subroutines for EVM Board APP001 in Mid Range HIGHTEC C Format
// Must include "MID_LCD.H"

#include "MID_LCD.h"
#include <pic.h>

//
// Defines for I/O ports that provide LCD data & control
// PORTD[0:3]-->DB[4:7]: Higher order 4 lines data bus with bidirectional
//					  : DB7 can be used as a BUSY flag
// PORTA.2 --> [E] : LCD operation start signal control 
// PORTD.5 --> [RW]: LCD Read/Write control
// PORTD.4 --> [RS]: LCD Register Select control
//		      	   : "0" for Instrunction register (Write), Busy Flag (Read)
//				   : "1" for data register (Read/Write)
//
#define 	CPU_SPEED			16				// CPU speed is 16 Mhz !!
#define 	LCD_RS				RD4			// The definition of control pins
#define 	LCD_RW				RD5
#define 	LCD_E				RA2
#define 	LCD_DATA			PORTD			// PORTD[0:3] as LCD DB[4:7]
#define 	DIR_LCD_DATA		TRISD
#define		MASK_DIR_LCD_DATA	0b00000000 
#define		DIR_LCD_RS			TRISD4
#define		DIR_LCD_RW			TRISD5
#define		DIR_LCD_E			TRISA2		


//  LCD Module commands
#define DISP_2Line_8Bit	0b00111000
#define DISP_2Line_4Bit	0b00101000
#define DISP_ON			0x00C		// Display on
#define DISP_ON_C		0x00E		// Display on, Cursor on
#define DISP_ON_B		0x00F		// Display on, Cursor on, Blink cursor
#define DISP_OFF		0x008		// Display off
#define CLR_DISP		0x001		// Clear the Display
#define ENTRY_INC		0x006		//
#define ENTRY_INC_S		0x007		//
#define ENTRY_DEC		0x004		//
#define ENTRY_DEC_S		0x005		//
#define DD_RAM_ADDR		0x080		// Least Significant 7-bit are for address
#define DD_RAM_UL		0x080		// Upper Left coner of the Display	

 
		
unsigned char 	Temp_CMD ;
unsigned char 	Str_Temp ;
unsigned char	Out_Mask ;	
unsigned char	Temp_LCD_DATA ;	

void OpenLCD(void)

{
	DIR_LCD_DATA = MASK_DIR_LCD_DATA ;	
	DIR_LCD_RS = 0 ;
	DIR_LCD_RW = 0 ;
	DIR_LCD_E = 0 ;

 	LCD_E=0;	
 	
 	TRISA2 = 0 ;						// RA2 as output								
	
	LCD_DATA = 0x00;					// LCD DB[4:7] & RS & R/W --> Low

	DIR_LCD_DATA = 0x00;				// LCD DB[4:7} & RS & R/W are output function
	LCD_E=0;					// Set E pin as output

	LCD_DATA = 0b00000011 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	LCD_DATA = 0b00000011 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	LCD_DATA = 0b00000011 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	LCD_DATA = 0b00000010 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	WriteCmdLCD(DISP_2Line_4Bit) ;
	LCD_S_Delay() ;

	WriteCmdLCD(DISP_ON) ;
	LCD_S_Delay() ;

	WriteCmdLCD(ENTRY_INC) ;
	LCD_S_Delay() ;

	WriteCmdLCD(CLR_DISP) ;
	LCD_L_Delay() ;

 		
}

//*********************************************
//     _    ______________________________
// RS  _>--<______________________________
//     _____
// RW       \_____________________________
//                  __________________
// E   ____________/                  \___
//     _____________                ______
// DB  _____________>--------------<______
//***********************************************
// Write Command to LCD module
//
void WriteCmdLCD( unsigned char LCD_CMD) 
{

	Temp_LCD_DATA = LCD_DATA ;	

	Temp_CMD = (LCD_CMD & 0xF0)>>4 ;			// Send high nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_CMD_W_Timing () ;

	Temp_CMD = LCD_CMD & 0x0F ;				// Send low nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_CMD_W_Timing () ;

	LCD_DATA = Temp_LCD_DATA ;

	LCD_S_Delay() ;							// Delay 100uS for execution
}

//***********************************************
// Write Data to LCD module
//
void WriteDataLCD( unsigned char LCD_CMD) 
{
	
	Temp_LCD_DATA = LCD_DATA ;

	Temp_CMD = (LCD_CMD & 0xF0)>>4 ;			// Send high nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_DAT_W_Timing () ;

	Temp_CMD = LCD_CMD & 0x0F ;				// Send low nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_DAT_W_Timing () ;

	LCD_DATA = Temp_LCD_DATA ;

	LCD_S_Delay() ;							// Delay 100uS for execution
}

void putcLCD(unsigned char LCD_Char)
{
	WriteDataLCD(LCD_Char) ;

}
void LCD_CMD_W_Timing( void )
{
	LCD_RS = 0 ;	// Set for Command Input
	#asm
		nop
		nop
	#endasm
	LCD_RW = 0 ;
 	#asm
 		nop
 		nop
 		nop
 		nop
 		nop
 		nop
 		nop
 		nop
 		nop
 		nop
 	#endasm
	LCD_E = 1 ;
	#asm
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
	#endasm
	LCD_E = 0 ;
}

void LCD_DAT_W_Timing( void )
{
	LCD_RS = 1 ;	// Set for Data Input
	
	#asm
		nop
		nop
	#endasm
	
	LCD_RW = 0 ;	
	
	#asm
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
	#endasm
	LCD_E = 1 ;
	#asm
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
	#endasm 
	LCD_E = 0 ;
}

//***********************************************
//     Set Cursor position on LCD module
//			CurY = Line (0 or 1)
//      	CurX = Position ( 0 to 15)
//
void LCD_Set_Cursor(unsigned char CurX, unsigned char CurY)
{
	WriteCmdLCD( 0x80 + CurY * 0x40 + CurX) ;
	LCD_S_Delay( ) ;
	LCD_S_Delay( ) ;
	LCD_S_Delay( ) ;
	LCD_S_Delay( ) ;
	LCD_S_Delay( ) ;
}

//***********************************************
//    Put a ROM string to LCD Module
//
void putrsLCD( const char *Str )
{
   while (1)
   {
	Str_Temp = *Str ;

		if (Str_Temp != 0x00 )
		   {
			WriteDataLCD(Str_Temp) ;
			Str ++ ;
		   }
		else
			break ;
   }
}

//***********************************************
//    Put a RAM string to LCD Module
//
void putsLCD( char *Str)
{
   while (1)
   {
	Str_Temp = *Str ;

		if (Str_Temp != 0x00 )
		   {
			WriteDataLCD(Str_Temp) ;
			Str ++ ;
		   }
		else
			break ;
   }
}


void puthexLCD(unsigned char HEX_Val)
{
	unsigned char Temp_HEX ;

	Temp_HEX = (HEX_Val >> 4) & 0x0f ;

	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;

	Temp_HEX = HEX_Val  & 0x0f ;
	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;
}

// *********************************************************************************
// Delay for atleast 10 ms 
// *********************************************************************************
void	LCD_L_Delay(void)
{
	unsigned int	Loop_L ;
	
	for ( Loop_L = 0 ; Loop_L < 10000 ; Loop_L ++ ) ;

}

// *********************************************************************************
// Delay for 100 us
// *********************************************************************************
void	LCD_S_Delay(void)
{
	unsigned int	Loop_S ;
		
	for ( Loop_S = 0 ; Loop_S < 100 ; Loop_S ++ ) ;	
}


void	Delay10KTCYx( int Loop_10K )
{
		int	Loop_1 , Loop_2 ;
		
		for ( Loop_1 = 0 ; Loop_1 < Loop_10K ; Loop_1++ ) 
		{
			for ( Loop_2 = 0 ; Loop_2 < 10000 ; Loop_2++ )
			{
				#asm
				 nop
				#endasm
			}
		}
		 
}

void	Delay100TCYx( int D_Value )
{
	int	D_Loop , D_LoopA ;
	
		for ( D_Loop = 0 ; D_Loop < D_Value ; D_Loop++ )
		{
			for (D_LoopA = 0 ; D_LoopA < 10 ; D_LoopA ++ ) 
			{
				#asm
			 	nop
				#endasm
			}
		}
}

void	LCD_Put_HEX(unsigned char HEX_Val)
{
	unsigned char Temp_HEX ;

	Temp_HEX = (HEX_Val >> 4) & 0x0f ;
	if ( Temp_HEX > 9 )
    	Temp_HEX += 0x37 ;
    else
		Temp_HEX += 0x30 ;

	putcLCD(Temp_HEX) ;

	Temp_HEX = HEX_Val  & 0x0f ;
	if ( Temp_HEX > 9 )
        Temp_HEX += 0x37 ;
    else
		Temp_HEX += 0x30 ;

	putcLCD(Temp_HEX) ;	

}
